@@ -1,7 +1,7 @@
 /*****************************************************************************\
   HPCupsFilter.cpp : Interface for HPCupsFilter class
 
-  Copyright (c) 1996 - 2009, Hewlett-Packard Co.
+  Copyright (c) 1996 - 2015, HP Co.
   All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
@@ -12,7 +12,7 @@
   2. Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
-  3. Neither the name of Hewlett-Packard nor the names of its
+  3. Neither the name of HP nor the names of its
      contributors may be used to endorse or promote products derived
      from this software without specific prior written permission.
 
@@ -31,6 +31,8 @@
 \*****************************************************************************/
 
 #include "HPCupsFilter.h"
+#include "ImageProcessor.h"
+
 #include <signal.h>
 #include <sys/wait.h>
 #include <sys/utsname.h>
@@ -41,6 +43,8 @@
 
 static HPCupsFilter    filter;
 
+
+#ifndef UNITTESTING 
 int main (int  argc, char *argv[])
 {
     openlog("hpcups", LOG_PID,  LOG_DAEMON);
@@ -52,6 +56,7 @@ int main (int  argc, char *argv[])
 
     return filter.StartPrintJob(argc, argv);
 }
+#endif
 
 void HPCancelJob(int sig)
 {
@@ -220,12 +225,18 @@ void HPCupsFilter::cleanup()
 {
     if (m_pPrinterBuffer) {
         delete [] m_pPrinterBuffer;
+        m_pPrinterBuffer = NULL;
     }
 
     if(m_ppd){
        ppdClose(m_ppd);
        m_ppd = NULL;
     }
+    if (m_pSys)
+    {
+    	delete m_pSys;
+    	m_pSys = NULL;
+    }
 }
 
 void HPCupsFilter::CancelJob()
@@ -341,6 +352,16 @@ DRIVER_ERROR HPCupsFilter::startPage (cups_page_header2_t *cups_header)
         m_JA.mech_offset = atoi(attr->value);
     }
 
+    if(cups_header->PageSize[0] > 612) // Check for B size paper. B Size paper has width > 8.5 inch (612 points)
+    {
+        //Check if HPMechOffsetBSize attribute is defined. If it is defined then use this offset value.
+        if (((attr = ppdFindAttr(m_ppd, "HPMechOffsetBSize", NULL)) != NULL) && (attr && attr->value != NULL)) 
+        {
+            m_JA.mech_offset = atoi(attr->value);
+        }
+
+    }
+
 //  Get printer platform name
     if (((attr = ppdFindAttr(m_ppd, "hpPrinterPlatform", NULL)) != NULL) && (attr->value != NULL)) {
 
@@ -366,6 +387,16 @@ DRIVER_ERROR HPCupsFilter::startPage (cups_page_header2_t *cups_header)
           m_JA.pre_process_raster = atoi(attr->value);
     }
 
+    if (((attr = ppdFindAttr(m_ppd, "HPSPDClass", NULL)) == NULL) ||
+         (attr && attr->value == NULL)) 
+    {
+        m_JA.HPSPDClass = 0;
+    }
+    else
+    {
+        m_JA.HPSPDClass = atoi(attr->value);
+    }
+    
 
 // Get the encapsulation technology from ppd
 
@@ -427,7 +458,10 @@ DRIVER_ERROR HPCupsFilter::startPage (cups_page_header2_t *cups_header)
     }
 
     string strPrinterURI="" , strPrinterName= "";
-    m_DBusComm.initDBusComm(DBUS_PATH,DBUS_INTERFACE, getenv("DEVICE_URI"), m_JA.printer_name);
+    if(!IsChromeOs()){
+      if (getenv("DEVICE_URI"))
+          m_DBusComm.initDBusComm(DBUS_PATH,DBUS_INTERFACE, getenv("DEVICE_URI"), m_JA.printer_name);
+    }
 
     ptr = strstr(m_argv[5], "job-uuid");
     if (ptr) {
@@ -484,7 +518,12 @@ int HPCupsFilter::StartPrintJob(int  argc, char *argv[])
     strncpy(m_JA.job_start_time, asctime(t), sizeof(m_JA.job_start_time)-1);    // returns Fri Jun  5 08:12:16 2009
     snprintf(m_JA.job_start_time+19, sizeof(m_JA.job_start_time) - 20, ":%ld %d", tv.tv_usec/1000, t->tm_year + 1900); // add milliseconds
 
-    getLogLevel();
+#ifdef UNITTESTING
+    memset(m_JA.job_start_time,0,sizeof(m_JA.job_start_time));
+    snprintf(m_JA.job_start_time, sizeof(m_JA.job_start_time),"Mon Dec  9 17:48:58:586 2013" );
+#endif
+
+    m_iLogLevel = getHPLogLevel();
     m_JA.job_id = atoi(argv[1]);
     strncpy(m_JA.user_name,argv[2],sizeof(m_JA.user_name)-1);
 
@@ -514,7 +553,7 @@ int HPCupsFilter::StartPrintJob(int  argc, char *argv[])
         }
     }
 
-    m_pSys = new SystemServices(m_iLogLevel, m_JA.job_id, m_JA.user_name);
+	m_pSys = new SystemServices(m_iLogLevel, m_JA.job_id, m_JA.user_name);
 
 /*
  *  When user cancels a print job, the spooler sends SIGTERM signal
@@ -594,14 +633,21 @@ int HPCupsFilter::processRasterData(cups_raster_t *cups_raster)
     DRIVER_ERROR           err;
     int                    ret_status = 0;
 
-    char hpPreProcessedRasterFile[64]; //temp file needed to store raster data with swaped pages.
+    char hpPreProcessedRasterFile[MAX_FILE_PATH_LEN]; //temp file needed to store raster data with swaped pages.
 
 
     sprintf(hpPreProcessedRasterFile, "%s/hp_%s_cups_SwapedPagesXXXXXX",CUPS_TMP_DIR, m_JA.user_name);
-
+    image_processor_t* imageProcessor = imageProcessorCreate();
 
     while (cupsRasterReadHeader2(cups_raster, &cups_header))
     {
+	    dbglog("DEBUG: SHIVA inside the while loop");
+
+        IMAGE_PROCESSOR_ERROR result = imageProcessorStartPage(imageProcessor, &cups_header);
+        if (result != IPE_SUCCESS){
+            dbglog("DEBUG: imageProcessorStartPage failed result = %d\n", result);
+        }
+
         current_page_number++;
 
         if (current_page_number == 1) {
@@ -610,16 +656,18 @@ int HPCupsFilter::processRasterData(cups_raster_t *cups_raster)
                 return JOB_CANCELED;
             }
 
-            if(m_JA.pre_process_raster) {
-		        err = m_Job.preProcessRasterData(&cups_raster, &cups_header, hpPreProcessedRasterFile);
-				if (err != NO_ERROR) {
-					if (m_iLogLevel & BASIC_LOG) {
-						dbglog ("DEBUG: Job::StartPage failed with err = %d\n", err);
-					}
-					ret_status = JOB_CANCELED;
-					break;
-				}
-			}
+            if (m_JA.pre_process_raster) {
+		    	// CC ToDo: Why pSwapedPagesFileName should be sent as a parameter? 
+                  	// Remove if not required to send it as parameter
+                err = m_Job.preProcessRasterData(&cups_raster, &cups_header, hpPreProcessedRasterFile);
+                if (err != NO_ERROR) {
+                    if (m_iLogLevel & BASIC_LOG) {
+                        dbglog ("DEBUG: Job::StartPage failed with err = %d\n", err);
+                    }
+                    ret_status = JOB_CANCELED;
+                    break;
+                }
+            }
 
             if (cups_header.cupsColorSpace == CUPS_CSPACE_RGBW) {
                 rgbRaster = new BYTE[cups_header.cupsWidth * 3];
@@ -660,14 +708,14 @@ int HPCupsFilter::processRasterData(cups_raster_t *cups_raster)
         // Save Raster file for Debugging
         if (m_iLogLevel & SAVE_INPUT_RASTERS)
         {
-            char    szFileName[64];
+            char    szFileName[MAX_FILE_PATH_LEN];
             memset(szFileName, 0, sizeof(szFileName));
 
             if (cups_header.cupsColorSpace == CUPS_CSPACE_RGBW ||
                 cups_header.cupsColorSpace == CUPS_CSPACE_RGB)
             {
 
-                snprintf (szFileName, sizeof(szFileName), "%s/hp_%s_cups_filterc_bmp_%d_XXXXXX", CUPS_TMP_DIR, m_JA.user_name, current_page_number);
+                snprintf (szFileName, sizeof(szFileName), "%s/hpcups_%s_c_bmp_%d_XXXXXX", CUPS_TMP_DIR, m_JA.user_name, current_page_number);
                 createTempFile(szFileName, &cfp);
                 if (cfp)
                 {
@@ -678,7 +726,7 @@ int HPCupsFilter::processRasterData(cups_raster_t *cups_raster)
             if (cups_header.cupsColorSpace == CUPS_CSPACE_RGBW ||
                 cups_header.cupsColorSpace == CUPS_CSPACE_K)
             {
-                snprintf (szFileName, sizeof(szFileName), "%s/hp_%s_cups_filterk_bmp_%d_XXXXXX", CUPS_TMP_DIR, m_JA.user_name, current_page_number);
+                snprintf (szFileName, sizeof(szFileName), "%s/hpcups_%s_k_bmp_%d_XXXXXX", CUPS_TMP_DIR, m_JA.user_name, current_page_number);
                 createTempFile(szFileName, &kfp);
                 if (kfp)
                 {
@@ -698,10 +746,16 @@ int HPCupsFilter::processRasterData(cups_raster_t *cups_raster)
             color_raster = rgbRaster;
             black_raster = kRaster;
 
+            result = imageProcessorProcessLine(imageProcessor, m_pPrinterBuffer, cups_header.cupsBytesPerLine);
+            if (result != IPE_SUCCESS){
+                dbglog("DEBUG: imageProcessorProcessLine failed result = %d\n", result);
+            }
+
+
             if ((y == 0) && !is_ljmono) {
                 //For ljmono, make sure that first line is not a blankRaster line.Otherwise printer
                 //may not skip blank lines before actual data
-                //Need to revisit to crosscheck if it is a firmware issue.
+                //Need to revisit to cross check if it is a firmware issue.
 
                 *m_pPrinterBuffer = 0x01;
                 dbglog("First raster data plane..\n" );
@@ -727,6 +781,12 @@ int HPCupsFilter::processRasterData(cups_raster_t *cups_raster)
             }
         }  // for() loop end
 
+        result = imageProcessorEndPage(imageProcessor);
+        if (result != IPE_SUCCESS){
+                dbglog("DEBUG: imageProcessorEndPage failed result = %d\n", result);
+        }
+
+
         m_Job.NewPage();
         if (err != NO_ERROR) {
             break;
@@ -737,8 +797,12 @@ int HPCupsFilter::processRasterData(cups_raster_t *cups_raster)
     if (cups_header.cupsColorSpace == CUPS_CSPACE_RGBW) {
         delete [] kRaster;
         delete [] rgbRaster;
+        kRaster = NULL;
+        rgbRaster = NULL;
     }
 
+    imageProcessorDestroy(imageProcessor);
+
     unlink(hpPreProcessedRasterFile);
     return ret_status;
 }
@@ -867,28 +931,3 @@ void HPCupsFilter::printCupsHeaderInfo(cups_page_header2_t *header)
     dbglog ("DEBUG: cupsReal0 = %f\n", header->cupsReal[0]); // Left overspray
     dbglog ("DEBUG: cupsReal1 = %f\n", header->cupsReal[1]); // Top overspray
 }
-
-void HPCupsFilter::getLogLevel ()
-{
-    FILE    *fp;
-    char    str[258];
-    char    *p;
-    fp = fopen ("/etc/cups/cupsd.conf", "r");
-    if (fp == NULL)
-        return;
-    while (!feof (fp))
-    {
-        if (!fgets (str, 256, fp))
-        {
-            break;
-        }
-        if ((p = strstr (str, "hpLogLevel")))
-        {
-            p += strlen ("hpLogLevel") + 1;
-            m_iLogLevel = atoi (p);
-            break;
-        }
-    }
-    fclose (fp);
-}
-
